// Copyright 2000-2023 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.intellij.ui

import com.intellij.openapi.Disposable
import com.intellij.openapi.wm.IdeGlassPane
import com.intellij.openapi.wm.impl.IdeGlassPaneImpl
import org.jetbrains.annotations.ApiStatus
import java.awt.Component
import java.awt.Cursor
import java.awt.Insets
import javax.swing.Icon
import javax.swing.JComponent

@ApiStatus.Internal
class WindowResizeListenerEx(private val glassPane: IdeGlassPane, content: Component, border: Insets, corner: Icon?) :
  WindowResizeListener(content, border, corner) {

  private val resizeListeners = mutableListOf<Runnable>()

  private var myCursor: Cursor? = null

  override fun setCursor(content: Component, cursor: Cursor) {
    if (myCursor !== cursor || myCursor !== Cursor.getDefaultCursor()) {
      glassPane.setCursor(cursor, this)
      myCursor = cursor
      if (content is JComponent) {
        IdeGlassPaneImpl.savePreProcessedCursor(content, content.getCursor())
      }
      super.setCursor(content, cursor)
    }
  }

  override fun notifyResized() {
    resizeListeners.forEach(Runnable::run)
  }

  fun install(parent: Disposable): WindowResizeListenerEx {
    glassPane.addMousePreprocessor(this, parent)
    glassPane.addMouseMotionPreprocessor(this, parent)
    return this
  }

  fun addResizeListeners(listener: Runnable) {
    resizeListeners.add(listener)
  }

  fun removeResizeListeners(listener: Runnable) {
    resizeListeners.remove(listener)
  }
}
